from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class EventSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'events'
        url = 'v4/events'

    id = fields.Int()
    status = fields.String()
    type = fields.String()
    title = fields.String()
    description = fields.String()
    image_url = fields.Url(data_key='image_url')
    promoted = fields.Boolean()
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    event_dates = fields.List(fields.Dict, data_key='eventDates')  # TODO
    app = fields.Function(get_id_from_url)
    venues = fields.List(fields.Inferred),  # TODO
    artists = fields.List(fields.Inferred),  # TODO
    useful_info = fields.List(fields.Inferred, data_key='usefulInfo'),  # TODO
    user_actions = fields.List(fields.Inferred, data_key='userActions'),  # TODO
    social_media = fields.List(fields.Inferred, data_key='socialMedia'),  # TODO
    translations = fields.List(fields.Inferred),  # TODO



from livestyled.schemas.app import AppSchema
from livestyled.schemas.device import DeviceSchema
from livestyled.schemas.device_consent import DeviceConsentSchema
from livestyled.schemas.device_token import DeviceTokenSchema
from livestyled.schemas.event import EventSchema
from livestyled.schemas.event_category import EventCategorySchema
from livestyled.schemas.event_integration import EventIntegrationSchema
from livestyled.schemas.event_stage import EventStageSchema
from livestyled.schemas.news import NewsSchema
from livestyled.schemas.push_consent import PushConsentSchema
from livestyled.schemas.team import TeamSchema
from livestyled.schemas.user import UserSchema
from livestyled.schemas.venue import VenueSchema

__all__ = [
    AppSchema,
    DeviceSchema,
    DeviceConsentSchema,
    DeviceTokenSchema,
    EventSchema,
    EventCategorySchema,
    EventIntegrationSchema,
    EventStageSchema,
    NewsSchema,
    PushConsentSchema,
    TeamSchema,
    UserSchema,
    VenueSchema,
]



from datetime import datetime, timedelta, timezone
import json
import os

import pytest

from livestyled import schemas
from livestyled.client import LiveStyledAPIClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


def test_deserialize_device():
    serialized_data = json.loads(open(os.path.join(FIXTURES_DIR, 'device_6.json'), 'r').read())
    deserialized_data = schemas.DeviceSchema().load(serialized_data)
    assert deserialized_data
    print(deserialized_data)
    assert deserialized_data['id'] == 6
    assert deserialized_data['token'] == '002332C86002'
    assert deserialized_data['consent'] == {
        'id': 2,
        'location_capture': True,
        'location_granular': None,
        'camera': True,
        'calendar': False,
        'photo_sharing': True,
        'push_notification': True,
        'created_at': datetime(2018, 12, 5, 17, 15, 3, tzinfo=timezone(timedelta(0), '+0000')),
        'updated_at': datetime(2018, 12, 5, 17, 15, 3, tzinfo=timezone(timedelta(0), '+0000'))
    }
    assert deserialized_data['push_consents'] == []
    assert deserialized_data['type'] == 'IOS'
    assert deserialized_data['status'] == 'ACTIVE'
    assert deserialized_data['app_version'] == '7.1'
    assert deserialized_data['os_version'] is None
    assert deserialized_data['model'] == 'Simulator'
    assert deserialized_data['manufacturer'] == 'Apple'
    assert deserialized_data['bluetooth_on'] is None
    assert deserialized_data['wifi_connected'] is None
    assert deserialized_data['created_at'] is None
    assert deserialized_data['updated_at'] is None


@pytest.fixture(scope='session')
def test_client():
    return LiveStyledAPIClient(
        api_domain=TEST_API_DOMAIN,
        api_key='thisisjustatest'
    )


def test_client_can_get_device_by_id(requests_mock, test_client):

    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/devices/1', 'mock_responses/ls_api/device_1.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    device = test_client.get_device(1)
    assert device == {
        'id': 1,
        'token': '002332C86002',
        'consent': {
            'id': 2,
            'location_capture': True,
            'location_granular': None,
            'camera': True,
            'calendar': False,
            'photo_sharing': True,
            'push_notification': True,
            'created_at': datetime(2018, 12, 5, 17, 15, 3, tzinfo=timezone(timedelta(0), '+0000')),
            'updated_at': datetime(2018, 12, 5, 17, 15, 3, tzinfo=timezone(timedelta(0), '+0000'))
        },
        'push_consents': [],
        'type': 'IOS',
        'status': 'ACTIVE',
        'app_version': '7.1',
        'os_version': None,
        'model': 'Simulator',
        'manufacturer': 'Apple',
        'bluetooth_on': None,
        'wifi_connected': None,
        'created_at': None,
        'updated_at': None,
    }


def test_client_can_get_list_of_devices(requests_mock, test_client):

    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/devices', 'mock_responses/ls_api/devices_list_page_1.json', 200),
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/devices?page=2', 'mock_responses/ls_api/devices_list_page_2.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    devices = test_client.get_devices()
    assert sum(1 for _ in devices) == 3


def test_client_can_update_device(requests_mock, test_client):
    mock_responses = (
        ('PATCH', 'https://' + TEST_API_DOMAIN + '/v4/devices/12345', 'mock_responses/ls_api/update_device_12345.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    test_client.update_device(
        12345,
        {
            'app_version': 'new-version',
            'bluetooth_on': True
        }
    )

    for request in requests_mock.request_history:
        if request.method.upper() == 'PATCH' and request.url == 'https://' + TEST_API_DOMAIN + '/v4/devices/12345':
            actual_payload = request.json()

            assert actual_payload == {'appVersion': 'new-version', 'bluetoothOn': True}



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class EventStageSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'event_stages'
        url = 'v4/event_stages'

    id = fields.Int()
    status = fields.String()
    venue = fields.Function(get_id_from_url)
    name = fields.String()
    description = fields.String()
    color = fields.String()
    sort_id = fields.Int(data_key='sortId')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)
    app = fields.Function(get_id_from_url)



from marshmallow import EXCLUDE, fields, Schema


class AppSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'apps'
        url = 'v4/apps'

    id = fields.Int()
    email = fields.Email(allow_none=True)
    roles = fields.List(fields.String(), allow_none=True)
    password = fields.String()
    salt = fields.String(allow_none=True)
    username = fields.String()
    code = fields.String()
    token = fields.String()
    name = fields.String()
    api_keys = fields.List(fields.String(), data_key='apiKeys')
    timezone = fields.String()
    google_api_key = fields.String(data_key='googleApiKey', allow_none=True)
    payment_gateway = fields.String(data_key='paymentGateway')
    venues = fields.List(fields.Inferred())  # TODO
    sender_email = fields.Email(data_key='senderEmail')
    deeplink_namespace = fields.String(data_key='deeplinkNamespace')
    merchant_accounts = fields.List(fields.Inferred, data_key='merchantAccounts', allow_none=True)
    title = fields.String()
    status = fields.String()
    currency = fields.Dict()  # TODO
    cohorts = fields.List(fields.Inferred)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class EventCategorySchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'event_categories'
        url = 'v4/event_categories'

    id = fields.Int()
    title = fields.String()
    resource = fields.String()
    force_show = fields.Boolean(data_key='forceShow')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)
    app = fields.Function(get_id_from_url)
    venue = fields.Function(get_id_from_url)
    sort_id = fields.Boolean(data_key='sortId')
    translations = fields.List(fields.Inferred, allow_none=True)



from marshmallow import EXCLUDE, fields, Schema


class NewsSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'news'
        url = 'v4/news'

    class NewsMediaSchema(Schema):
        class Meta:
            unknown = EXCLUDE

        type = fields.String(required=True, allow_none=False)

    id = fields.Int()
    external_id = fields.String(data_key='externalId')
    headline = fields.String(data_key='headline')
    image_url = fields.URL(data_key='imageURL')
    title = fields.String(data_key='title')
    media = fields.Nested(NewsMediaSchema, required=True)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class DeviceTokenSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'device_tokens'
        url = 'v4/device_tokens'

    id = fields.Int()
    provider = fields.String()
    provider_token = fields.String(data_key='providerToken')
    payload = fields.Dict(keys=fields.String(), allow_none=True)
    device = fields.Function(deserialize=get_id_from_url)



from marshmallow import EXCLUDE, fields, Schema


class DeviceConsentSchema(Schema):
    class Meta:
        unknown = EXCLUDE

    id = fields.Int()
    location_capture = fields.Boolean(data_key='locationCapture', allow_none=True)
    location_granular = fields.Boolean(data_key='locationGranular', allow_none=True)
    camera = fields.Boolean(data_key='camera', allow_none=True)
    calendar = fields.Boolean(allow_none=True)
    photo_sharing = fields.Boolean(data_key='photoSharing', allow_none=True)
    push_notification = fields.Boolean(data_key='pushNotification', allow_none=True)
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)



import os

import pytest

from livestyled.client import LiveStyledAPIClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


@pytest.fixture(scope='session')
def test_client():
    return LiveStyledAPIClient(
        api_domain=TEST_API_DOMAIN,
        api_key='thisisjustatest'
    )


def test_client_can_get_app_by_id(requests_mock, test_client):

    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/apps/1', 'mock_responses/ls_api/app_1.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    app = test_client.get_app(1)
    assert app == {
        'email': None,
        'roles': [
            'ROLE_APP'
        ],
        'password': 'APPID',
        'salt': None,
        'username': 'TestAppUser1',
        'code': 'TARENA1',
        'token': 'token',
        'id': 1,
        'name': 'Test Arena 1',
        'api_keys': [
            'APPID'
        ],
        'timezone': 'Europe/London',
        'google_api_key': None,
        'payment_gateway': 'PAYMENTGW1',
        'venues': [],
        'sender_email': 'support@livestyed.net',
        'deeplink_namespace': '',
        'merchant_accounts': [],
        'title': 'Test Arena 1',
        'status': 'DISABLED',
        'currency': {
            'id': 1,
            'title': 'Pound',
            'isoCode': 'GBP',
            'sign': '£',
            'updatedAt': '2016-08-09T10:49:34+00:00',
            'createdAt': '2016-08-09T10:49:34+00:00',
            '__initializer__': None,
            '__cloner__': None,
            '__isInitialized__': True
        },
        'cohorts': []
    }


def test_client_can_get_list_of_apps(requests_mock, test_client):

    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/apps', 'mock_responses/ls_api/apps_list_page_1.json', 200),
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/apps?page=2', 'mock_responses/ls_api/apps_list_page_2.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    apps = test_client.get_apps()
    assert sum(1 for _ in apps) == 4



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class VenueSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'venues'
        url = 'v4/venues'

    id = fields.Int()
    label = fields.String()
    name = fields.String()
    status = fields.String()
    is_default = fields.Boolean(data_key='is_default')
    useful_info = fields.List(fields.Inferred, data_key='usefulInfo')  # TODO
    event_useful_info = fields.List(fields.Inferred, data_key='eventUsefulInfo')  # TODO
    app = fields.Function(get_id_from_url)
    social_media = fields.List(fields.Inferred, data_key='socialMedia')  # TODO



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class PushConsentSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'push_consents'
        url = 'v4/push_consents'

    id = fields.Int()
    type = fields.String()
    label = fields.String()
    title = fields.String()
    sort_id = fields.Int(data_key='sortId')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)
    app = fields.Function(get_id_from_url)
    translations = fields.Dict()  # TODO



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class UserSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'users'
        url = 'v4/users'

    id = fields.Int()
    customer = fields.Dict()  # TODO
    user_info = fields.Dict(data_key='userInfo')  # TODO
    auth_source = fields.String(data_key='authSource')
    token = fields.String()
    auth_type = fields.String(data_key='authType')
    type = fields.String()
    first_name = fields.String(data_key='firstName')
    last_name = fields.String(data_key='lastName')
    status = fields.String()
    roles = fields.List(fields.Inferred)  # TODO
    username = fields.String()
    username_canonical = fields.String(data_key='usernameCanonical')
    salt = fields.String()
    email = fields.Email
    email_canonical = fields.Email(data_key='emailCanonical')
    password = fields.String()
    last_login = fields.AwareDateTime(data_key='lastLogin', allow_none=True)
    account_non_expired = fields.Boolean(data_key='accountNonExpired')
    account_non_locked = fields.Boolean(data_key='accountNonLocked')
    credentials_non_expired = fields.Boolean(data_key='credentialsNonExpired')
    enabled = fields.Boolean()
    super_admin = fields.Boolean(data_key='superAdmin')
    groups = fields.List(fields.Inferred)  # TODO
    group_names = fields.List(fields.Inferred, data_key='groupNames')  # TODO
    app = fields.Function(get_id_from_url)
    devices = fields.List(fields.Inferred)  # TODO
    user_consent = fields.Dict(data_key='userConsent')  # TODO
    user_push_consents = fields.List(fields.Inferred, data_key='userPushConsents')
    tickets = fields.List(fields.Inferred)
    sso_user = fields.Boolean(data_key='SSOUser')
    user_tokens = fields.List(fields.Inferred, data_key='userTokens')
    cohorts = fields.List(fields.Inferred)
    user_emails = fields.List(fields.Inferred, data_key='userEmails')  # TODO
    magic_fields = fields.List(fields.Inferred, data_key='magicFields')  # TODO
    principal_user_email = fields.Dict(data_key='principalUserEmail')  # TODO
    gender = fields.String()



from marshmallow import EXCLUDE, fields, Schema


class DeviceSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'devices'
        url = 'v4/devices'

    id = fields.Int()
    token = fields.String()
    consent = fields.Nested('livestyled.schemas.device_consent.DeviceConsentSchema', allow_none=True)
    push_consents = fields.List(fields.Dict, data_key='pushConsents', allow_none=True)  # TODO
    type = fields.String()
    status = fields.String()
    app_version = fields.String(data_key='appVersion')
    os_version = fields.String(data_key='osVersion', allow_none=True)
    model = fields.String(allow_none=True)
    manufacturer = fields.String(allow_none=True)
    bluetooth_on = fields.Boolean(data_key='bluetoothOn', allow_none=True)
    wifi_connected = fields.Boolean(data_key='wifiConnected', allow_none=True)
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)



from marshmallow import EXCLUDE, fields, Schema

from livestyled.schemas.utils import get_id_from_url


class EventIntegrationSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'event_integration'
        url = 'v4/event_integration'

    id = fields.Int()
    status = fields.String()
    endpoint_url = fields.Url(data_key='endpointUrl')
    timezone = fields.String()
    adapter = fields.String()
    config_payload = fields.String(data_key='configPayload')
    updated_at = fields.AwareDateTime(data_key='updatedAt', allow_none=True)
    created_at = fields.AwareDateTime(data_key='createdAt', allow_none=True)
    app = fields.Function(get_id_from_url)
    remove_old = fields.Boolean(data_key='removeOld')
    override_social_media = fields.Boolean(data_key='overrideSocialMedia')



from marshmallow import EXCLUDE, fields, Schema


class TeamSchema(Schema):
    class Meta:
        unknown = EXCLUDE
        api_type = 'teams'
        url = 'v4/teams'

    id = fields.Int()
    name = fields.String(data_key='name')
    short_name = fields.String(data_key='shortName')
    light_crest_url = fields.URL(data_key='lightCrestURL', allow_none=True)
    dark_crest_url = fields.URL(data_key='darkCrestURL')
    external_id = fields.String(data_key='externalId')



import os


def configure_mock_responses(requests_mock, mock_responses, fixtures_dir, content_type):
    for method, url, fixture, status_code in mock_responses:
        data = open(os.path.join(fixtures_dir, fixture), 'r').read()
        if method == 'GET':
            requests_mock.get(url, text=data, headers={'Content-Type': content_type}, status_code=status_code)
        elif method == 'POST':
            requests_mock.post(url, text=data, headers={'Content-Type': content_type}, status_code=status_code)
        elif method == 'PATCH':
            requests_mock.patch(url, text=data, headers={'Content-Type': content_type}, status_code=status_code)
